@@ -1,3 +1,6 @@
 # -*- coding: utf-8 -*-
 from .czjzek import CzjzekDistribution  # noqa: F401
 from .czjzek import ExtCzjzekDistribution  # noqa: F401
+
+__author__ = "Deepansh J. Srivastava"
+__email__ = "srivastava.89@osu.edu"
