@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 
+__author__ = "Deepansh Srivastava"
+__email__ = "srivastava.89@osu.edu"
+
 
 def get_spectral_dimensions(csdm_object):
     """
@@ -28,6 +31,8 @@ def get_spectral_dimensions(csdm_object):
         else:
             dim_i["reference_offset"] = coordinates_offset + spectral_width / 2.0
 
+        if dim.label != "":
+            dim_i["label"] = dim.label
         result.append(dim_i)
 
-    return result
+    return result[::-1]
